from threading import Thread
from asgiref.sync import async_to_sync
import time
from django.template.loader import render_to_string
from .context_processors import get_global_context
from asgiref.sync import sync_to_async
from django.utils.translation import activate as translation_activate
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from uuid import uuid4
import base64
from django.core.files import File
from tempfile import NamedTemporaryFile


async def get_html(template, context={}):
    """Get html from template."""
    return await sync_to_async(render_to_string)(template, context)


def set_language(language="en"):
    """Set the language."""
    if language:
        translation_activate(language)


def enable_lang(func):
    """Decorator: Enable language"""

    def wrapper(*args, **kwargs):
        lang = args[1]["data"].get("lang", settings.LANGUAGE_CODE)
        set_language(lang)
        kwargs["lang"] = lang
        return func(*args, **kwargs)

    return wrapper


async def toggle_loading(consumer, show=False):
    """Toogle the footer form."""
    # html = await get_html(template, get_global_context(consumer=consumer))
    data = {
        "action": ("Show" if show else "Hide") + " loading",
        "selector": "#loading",
        "html": render_to_string(
            "components/_loading.html", get_global_context(consumer=consumer)
        )
        if show
        else "",
    }
    await consumer.send_html(data)


def loading(func):
    """Decorator: Show loading."""

    async def wrapper(*args, **kwargs):
        await toggle_loading(args[0], True)
        result = await func(*args, **kwargs)
        await toggle_loading(args[0], False)
        return result

    return wrapper


async def update_active_nav(consumer, page):
    """Update the active nav item in the navbar."""
    context = get_global_context(consumer=consumer)
    context["active_nav"] = page
    data = {
        "action": "Update active nav",
        "selector": "#content-header",
        "html": render_to_string("components/_header.html", context),
    }
    await consumer.send_html(data)


def send_email(
    subject="", to=[], template_txt="", template_html="", data={}, attachments=[]
):
    """Send email"""
    msg = EmailMultiAlternatives(
        subject,
        render_to_string(template_txt, data | {"settings": settings}),
        settings.DEFAULT_FROM_EMAIL,
        to,
    )
    msg.attach_alternative(
        render_to_string(template_html, data | {"settings": settings}), "text/html"
    )
    for attachment in attachments:
        msg.attach_file(attachment)
    return msg.send()


async def send_notification(consumer: object, message: str, level: str = "info"):
    """Send notification."""
    # Variables
    uuid = str(uuid4())
    timeout = 3000  # ms

    async def make_notification(consumer=None, uuid="", level="", message=""):
        # Show message
        context = get_global_context(consumer=consumer)
        context.update({"id": uuid, "message": message, "level": level})
        html = await get_html("components/_notification.html", context)
        data = {
            "action": "new_notification",
            "selector": "#notifications",
            "html": html,
            "append": True,
        }
        await consumer.send_html(data)

    # Remove message async
    def remove_notification(consumer=None, uuid="", timeout=0):
        time.sleep(timeout / 1000)
        data = {
            "action": "delete_notification",
            "selector": f"#notifications > #notifications__item-{uuid}",
            "html": "",
        }
        async_to_sync(consumer.send_html)(data)

    # Tasks
    await make_notification(consumer, uuid, level, message)
    # Run in background the remove notification, sleep 3 seconds
    Thread(target=remove_notification, args=(consumer, uuid, timeout)).start()


def get_image_from_base64(base64_string: str, mime_type: str, is_file: bool = True):
    """Get image from base64 string.
    Args:
    base64_string (str): Base64 string.
    mime_type (str): Mime type. Example: image/jpeg.
    is_file (bool): Return a file or bytes.

    Returns:
    File or bytes: Image.
    str: Filename.
    """
    if mime_type in (
        "image/jpeg",
        "image/png",
        "image/webp",
    ):
        # Variables
        uuid = str(uuid4())
        extension = mime_type.split("/")[-1]
        # Str base64 to bytes
        base64_img_bytes = base64_string.encode("utf-8")
        decoded_image_data = base64.decodebytes(base64_img_bytes)
        my_filename = f"{uuid}.{extension}"
        if is_file:
            # Bytes to file
            img_temp = NamedTemporaryFile(delete=True)
            img_temp.write(decoded_image_data)
            img_temp.flush()
            return File(img_temp), my_filename
        else:
            return decoded_image_data, my_filename
    return None, None



import os
import sys
import threading
from django.conf import settings
from channels.generic.websocket import AsyncJsonWebsocketConsumer
from channels.db import database_sync_to_async
from .models import Client
from asgiref.sync import async_to_sync, sync_to_async
from channels.layers import get_channel_layer
from django.template.loader import render_to_string


# Load all actions from settings.LIVEVIEW_APPS
for current_app in settings.LIVEVIEW_APPS:
    for entry in os.scandir(os.path.join(settings.BASE_DIR, current_app.replace(".", "/"), "actions")):
        if entry.is_file():
            name = entry.name.split(".")[0]
            exec(f"from {current_app}.actions import {name} as {name}")


class LiveViewConsumer(AsyncJsonWebsocketConsumer):

    channel_name_broadcast = "broadcast"

    @database_sync_to_async
    def create_client(self, channel_name):
        Client.objects.create(channel_name=channel_name)

    @database_sync_to_async
    def delete_client(self, channel_name):
        Client.objects.filter(channel_name=channel_name).delete()

    async def connect(self):
        """Event when client connects"""
        # Accept the connection
        await self.accept()
        # Add to group broadcast
        await self.channel_layer.group_add(
            self.channel_name_broadcast, self.channel_name
        )
        # Save the client
        await self.create_client(self.channel_name)

    async def disconnect(self, close_code):
        """Event when client disconnects"""
        # Remove from group broadcast
        await self.channel_layer.group_discard(
            self.channel_name_broadcast, self.channel_name
        )
        # Delete the client
        await self.delete_client(self.channel_name)

    async def receive_json(self, data_received):
        """
        Event when data is received
        All information will arrive in 2 variables:
        "action", with the action to be taken
        "data" with the information
        """
        # Get the data
        data = data_received if "data" in data_received else None
        # Depending on the action we will do one task or another.
        # Example: If the action is "home->search", we will call the function "actions.home.search" with the data
        if data and "action" in data:
            action_data = data["action"].split("->")
            if len(action_data) == 2:
                action = action_data[0].lower()
                function = action_data[1].lower()
                try:
                    await eval(f"{action}.{function}(self, data)")
                except Exception as e:
                    print(f"Bad action: {data['action']}")
                    # Print the error
                    exc_type, exc_obj, exc_tb = sys.exc_info()
                    print(exc_type)

    async def send_html(self, data, broadcast=False):
        """Event: Send html to client

        Example minimum data:
        {
            "action": "home->search",
            "selector": "#search-results",
            "html": "<h1>Example</h1>"
        }

        Example with optional data:
        {
            "action": "home->search",
            "selector": "#search-results",
            "html": "<h1>Example</h1>",
            "append": true, # Optional, default: false. If true, the html will be added, not replaced
            "url": "/search/results", # Optional, default: None. If set, the url will be changed
            "title": "Search results", # Optional, default: None. If set, the title will be changed
            "scroll": true # Optional, default: false. If true, the page will be scrolled to the selector
            "scrollTop": false # Optional, default: false. If true, the page will be scrolled to the top
        }
        """
        if "selector" in data and "html" in data:
            # Required data
            my_data = {
                "action": data["action"],
                "selector": data["selector"],
                "html": data["html"],
            }
            # Optional data
            if "append" in data:
                my_data.update({"append": data["append"]})
            else:
                my_data.update({"append": False})
            if "url" in data:
                my_data.update({"url": data["url"]})
            if "title" in data:
                my_data.update({"title": data["title"]})
            if "scroll" in data:
                my_data.update({"scroll": data["scroll"]})
            if "scrollTop" in data:
                my_data.update({"scrollTop": data["scrollTop"]})
            # Send the data
            if broadcast:
                if hasattr(self, "channel_layer"):
                    await self.channel_layer.group_send(
                        self.channel_name_broadcast,
                        {"type": "send_data_to_frontend", "data": my_data},
                    )
            else:
                await self.send_data_to_frontend(my_data)

    async def send_data_to_frontend(self, data):
        """Send data to the frontend"""
        # Corrects the data if it comes from an external call or a group_send
        send_data = data["data"] if "type" in data else data
        await self.send_json(send_data)



from django.conf import settings
from channels.auth import get_user
from asgiref.sync import async_to_sync


def get_global_context(consumer=None):
    """Return a dictionary of global context variables."""
    return {
        "DEBUG": settings.DEBUG,
        "user": consumer.scope["user"]
        if consumer and "user" in consumer.scope
        else None,
    }


def customs(request):
    """Return a dictionary of context variables."""
    context = get_global_context()
    # Fix for admin site
    context.pop("user")
    return context



from django.db import models
from django.contrib.auth.models import User
from django.conf import settings
from django.utils.text import slugify


class Client(models.Model):
    """
    Each client who is connected to the website
    """

    user = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
    channel_name = models.CharField(max_length=200, blank=True, null=True, default=None)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.user.username if self.user else self.channel_name



from setuptools import setup

setup(
    name="django-liveview",
    py_modules=["liveview"],
    version="1.0.4",
    python_requires=">3.7",
    description="Framework for creating Realtime SPAs using HTML over the Wire technology in Django",
    author="Andros Fenollosa",
    author_email="andros@fenollosa.email",
    url="https://django-liveview.andros.dev/",
    license="MIT License",
    platforms=['any'],
    packages=["liveview"],
    keywords=["django", "ssr", "channels", "liveview", "html-over-the-wire", "hotwire"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    install_requires=["channels", "django", "channels_redis"],
    entry_points="",
)
